#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 2A.ml - 2017 - Préparation des données
# 
# Ce notebook explique comment les données de la compétation 2017 ont été préparées. On récupére d'abord les données depuis le site [OpenFoodFacts](https://world.openfoodfacts.org/data).

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## A quoi ça ressemble

# In[2]:


import os
os.stat("c:/temp/fr.openfoodfacts.org.products.csv").st_size / 2**30, 'Go'


# C'est gros.

# In[3]:


import pyensae


# In[4]:


get_ipython().run_line_magic('load_ext', 'pyensae')


# In[5]:


get_ipython().run_line_magic('head', '-n 2 c:/temp/fr.openfoodfacts.org.products.csv')


# In[6]:


import pandas


# In[7]:


df = pandas.read_csv("c:/temp/fr.openfoodfacts.org.products.csv", 
                     sep="\t", encoding="utf-8", nrows=10000, low_memory=False)


# In[8]:


df.head().T.to_excel("e.xlsx")


# In[9]:


df[df.additives.notnull() & df.additives.str.contains("E4")].head().T


# ## Idée de la compétation
# 
# On veut savoir les additifs ajoutés apparaissent plus fréquemment avec certains produits ou certains compositions. ON cherche donc à prédire la présence d'additifs en fonction de toutes les autres variables. Si un modèle de prédiction fait mieux que le hasard, cela signifie que certaines corrélations existent. J'ai utilisé [dask](https://dask.pydata.org/en/latest/) mais si vous de la mémoire, on peut faire avec [pandas](https://pandas.pydata.org/).

# In[10]:


import dask
import dask.dataframe as dd


# Le code qui suit est construit après plusieurs essais en fonction des warnings retournés par le module *dask*.

# In[11]:


ddf = dd.read_csv("c:/temp/fr.openfoodfacts.org.products.csv", sep="\t", encoding="utf-8", low_memory=False,
                 dtype={'allergens': 'object',
       'cities_tags': 'object',
       'emb_codes': 'object',
       'emb_codes_tags': 'object',
       'first_packaging_code_geo': 'object',
       'generic_name': 'object',
       'ingredients_from_palm_oil_tags': 'object',
       'labels': 'object',
       'labels_fr': 'object',
       'labels_tags': 'object',
       'manufacturing_places': 'object',
       'manufacturing_places_tags': 'object',
       'origins': 'object',
       'origins_tags': 'object',
       'stores': 'object',
       'code': 'object','allergens_fr': 'object',
       'cities': 'object',
       'created_t': 'object',
       'last_modified_t': 'object'})
ddf.head()


# In[12]:


print(type(ddf))


# On ajoute la colonne à prédire, booleénne, qui indique la présence d'additif commençant par ``'e:'`` comme [E440](http://www.les-additifs-alimentaires.com/E440-pectines.phphttp://www.les-additifs-alimentaires.com/E440-pectines.php).

# In[13]:


ddfe = ddf.assign(hasE=ddf.apply(lambda row: isinstance(row.additives, str) and "en:e" in row.additives, 
                                 axis=1, meta=bool))
ddfe.head()


# On se limite au produit pour lesquels on a quelques informations sur le contenu.

# In[14]:


g100 = [_ for _ in ddf.columns if '100g' in _]
g100


# In[15]:


ddfe.compute().shape


# In[16]:


import numpy

ddfe100 = ddfe.assign(s100=ddf.apply(lambda row: sum(0 if numpy.isnan(row[g]) else 1 for g in g100), 
                                     axis=1, meta=float))
ddfe100 = ddfe100[ddfe100.s100 > 0]
ddfe100.head()


# Bon la suite prend un peu de temps et ça n'est pas hyper efficace. Il faudrait un dask qui n'utilise pas dask mais uniquement les dataframes pour que ça aille plus vite. Café.

# In[17]:


ddfe100.to_csv("ddfe100*.csv", sep="\t", encoding="utf-8", index=False)


# Bon je crois que je vais vraiment développer une truc comme dask juste avec pandas.

# In[18]:


dffefiles = [_ for _ in os.listdir(".") if "ddfe" in _]
dffefiles


# ## Split...
# 
# On impose les mêmes types pour chaque data frame.

# In[19]:


types = {k:v for k, v in zip(ddfe100.columns, ddfe100.dtypes)}


# In[20]:


from sklearn.model_selection import train_test_split

for i, name in enumerate(dffefiles):
    print("name", name)
    df = pandas.read_csv(name, sep="\t", encoding="utf-8", dtype=types)
    df_train, df_test = train_test_split(df, test_size =0.5)
    df_test, df_eval = train_test_split(df_test, test_size =0.5)
    df_train.to_csv("off_train{0}.txt".format(i), sep="\t", index=False, encoding="utf-8")
    df_test.to_csv("off_test{0}.txt".format(i), sep="\t", index=False, encoding="utf-8")
    df_eval.to_csv("off_eval{0}.txt".format(i), sep="\t", index=False, encoding="utf-8")


# Ah j'allais oublié, il faut bidouiller la colonne *additives* pour retirer éviter un memory leak et on recalcule la colonne hasE pour être sûr.

# In[21]:


df[["additives", "hasE"]].head()


# In[22]:


import re
reg = re.compile("[[](.*?)[]]")
addi = re.compile("(en[:]e[0-9])")


# In[23]:


def has_emachine(v):
    if isinstance(v, (list, pandas.core.series.Series)):
        rem = []
        add = []
        for _ in v:
            if isinstance(_, str):
                fd = reg.findall(_)
                for __ in fd:
                    if " en:e" in __ and addi.search(__):                      
                        add.append(__)#.split("->")[-1].strip())
                    elif " en:" not in __:
                        continue
                    else:
                        rem.append(__.split("->")[-1].strip())
            else:
                continue
        return add, list(sorted(set(rem)))
    elif isinstance(v, float) and numpy.isnan(v):
        return [], []
    elif isinstance(v, str):
        if "," in v:
            raise Exception('{0}\n{1}'.format(type(v), v))
        return has_emachine([v])
    else:
        # ???
        raise Exception('{0}\n{1}'.format(type(v), v))
                    
hasE, clean = has_emachine(df.loc[1,"additives"])
hasE, clean


# On recompose le tout.

# In[24]:


off = [_ for _ in os.listdir(".") if "off" in _ and "all" not in _]

for cont in ['train', 'test', 'eval']:
    sub = [_ for _ in off if cont in _]
    dfs = []
    for name in sub:
        df = pandas.read_csv(name, sep="\t", encoding="utf-8", dtype=types)
        print("name", name, df.shape)
        df["hasE"] = df["additives"].apply(lambda x: len(has_emachine(x)[0]) > 0)
        df["additives"] = df["additives"].apply(lambda x: ";".join(has_emachine(x)[1]))
        dfs.append(df)
    df = pandas.concat(dfs, axis=0)
    print("merged", df.shape)
    df.to_csv("off_{0}_all.txt".format(cont), sep="\t", index=False, encoding="utf-8")


# Il y aura probablement un ou deux data leak dans les autres colonnes..

# On découpe le jeu d'évaluation.

# In[25]:


len(types)


# In[26]:


df_eval = pandas.read_csv("off_eval_all.txt", sep="\t", dtype=types, encoding="utf-8")


# In[27]:


df_eval_X = df_eval.drop("hasE", axis=1)
df_eval_X.to_csv("off_eval_all_X.txt")
df_eval[["hasE"]].to_csv("off_eval_all_Y.txt")


# ## Premier modèle

# In[28]:


df_train = pandas.read_csv("off_train_all.txt", sep="\t", dtype=types, encoding="utf-8")


# In[29]:


df_train.shape


# In[30]:


X = df_train[g100].fillna(0)
Y = df_train['hasE']


# In[31]:


from sklearn.linear_model import LogisticRegression
clf = LogisticRegression()


# In[32]:


clf.fit(X, Y)


# In[33]:


pred = clf.predict(X)


# In[34]:


from sklearn.metrics import confusion_matrix
confusion_matrix(Y, pred)


# In[35]:


df_test = pandas.read_csv("off_test_all.txt", sep="\t", dtype=types, encoding="utf-8")


# In[36]:


X_test = df_test[g100].fillna(0)
Y_test = df_test['hasE']


# In[37]:


pred = clf.predict(X_test)
confusion_matrix(Y_test, pred)


# ## ROC

# In[38]:


y_proba = clf.predict_proba(X_test)
y_pred = clf.predict(X_test)
print(y_proba[:3])
print(y_pred[:3])


# In[39]:


y_test = Y_test.values
type(y_pred), type(Y_test), type(y_test)


# In[40]:


import numpy
prob_pred = numpy.array([(y_proba[i, 1] if c else y_proba[i, 0]) for i, c in enumerate(y_pred)])
prob_pred[:3]


# In[41]:


from sklearn.metrics import roc_curve
fpr, tpr, th = roc_curve(y_pred == y_test, prob_pred)


# In[42]:


get_ipython().run_line_magic('matplotlib', 'inline')
import matplotlib.pyplot as plt
plt.figure()
lw = 2
plt.plot(fpr, tpr, color='darkorange', lw=lw, label='Courbe ROC')
plt.plot([0, 1], [0, 1], color='navy', lw=lw, linestyle='--')
plt.xlim([0.0, 1.0])
plt.ylim([0.0, 1.05])
plt.xlabel("Proportion mal classée")
plt.ylabel("Proportion bien classée")
plt.title('ROC')
plt.legend(loc="lower right")


# Bon c'est un modèle linéaire donc je suis sûr que vous ferez mieux et puis il y a le pays, la date, les autres ingrédients, bref pas mal de texte.

# In[43]: